@@ -158,6 +158,18 @@ class StreamsClient(ConnectionManager[RabbitMQStreamsProtocol]):
         protocol = await self.get_protocol()
         await protocol.create_stream(stream)
 
+    async def delete_stream(self, stream: str) -> None:
+        """
+        Delete RabbitMQ stream.
+
+        Method ignores error received from RabbitMQ Streams broker if the
+        stream does not exist.
+
+        :param stream: RabbitMQ stream name.
+        """
+        protocol = await self.get_protocol()
+        await protocol.delete_stream(stream)
+
     #
     # let's hack API for publisher; the goal is to avoid mixing concepts
     # within single publisher class
