@@ -22,8 +22,10 @@ VERSION = 1
 DEFAULT_FRAME_SIZE = 1024 ** 2 # 1 MB
 DEFAULT_HEARTBEAT = 60  # in seconds
 
-# TODO: make this configurable
-DEFAULT_CREDIT = 128
+# TODO: make default credit and max queue size configurable per
+# subscription
+DEFAULT_CREDIT = 32
+QUEUE_SIZE_THRESHOLD = 1024
 
 KEY_DECLARE_PUBLISHER = 0x01
 KEY_PUBLISH = 0x02
@@ -38,6 +40,7 @@ KEY_STORE_OFFSET = 0x0a
 KEY_QUERY_OFFSET = 0x0b
 KEY_UNSUBSCRIBE = 0x0c
 KEY_CREATE_STREAM = 0x0d
+KEY_DELETE_STREAM = 0x0e
 KEY_METADATA_QUERY = 0x0f
 KEY_PEER_PROPERTIES = 0x11
 KEY_SASL_HANDSHAKE = 0x12
