@@ -59,15 +59,16 @@ import asyncio
 import dataclasses as dtc
 import logging
 import typing as tp
-from collections import defaultdict
+from collections import defaultdict, deque
 
 from ..amqp import MessageCtx
 from ..util import Option
 from . import codec
 from . import const
+from ._mqueue import MessageQueue
 from .error import ProtocolError
 from .offset import Offset, OffsetType
-from .types import MessageQueue, PublishError
+from .types import PublishError
 from .util import concatv, retry
 
 logger = logging.getLogger(__name__)
@@ -99,6 +100,7 @@ REQUESTS_KEYS = {
     const.RESPONSE_KEY_DELETE_PUBLISHER,   # type: ignore
     const.RESPONSE_KEY_SUBSCRIBE,          # type: ignore
     const.RESPONSE_KEY_CREATE_STREAM,      # type: ignore
+    const.RESPONSE_KEY_DELETE_STREAM,      # type: ignore
     const.RESPONSE_KEY_QUERY_PUBLISHER_SEQUENCE,  # type: ignore
 }
 # pylint: enable=no-member
@@ -114,11 +116,14 @@ class SentMessages:
 
 @dtc.dataclass(frozen=True, slots=True)
 class ReceivedMessages:
-    task: asyncio.Future[None]
-    offset: tp.Optional[int]
     queue: MessageQueue
+    offset: tp.Optional[int]
     amqp: bool
 
+    @property
+    def task(self) -> asyncio.Future[None]:
+        return self.queue.task
+
 class RabbitMQStreamsProtocol(asyncio.Protocol):
     """
     Asyncio protocol for RabbitMQ streams.
@@ -218,6 +223,22 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         # always send metadata query for a stream
         return await self.query_stream_metadata(stream)
 
+    async def delete_stream(self, stream: str) -> tp.Any:
+        """
+        Delete RabbitMQ stream.
+
+        :param stream: Stream name.
+        """
+        try:
+            await self.send_delete_stream(stream)
+        except ProtocolError as ex:
+            if ex.code == 6:
+                logger.info('rabbitmq stream does not exist: {}'.format(stream))
+            else:
+                raise
+        else:
+            logger.info('rabbitmq stream deleted: {}'.format(stream))
+
     @retry(ProtocolError, predicate=is_stream_na, retry_after=1)
     async def create_publisher(
         self, publisher_id: int, publisher_ref: str, stream: str
@@ -334,29 +355,30 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
 
         task = self._loop.create_future()
         data = codec.encode_subscribe(sid, stream, offset, const.DEFAULT_CREDIT)
-        self._subscriptions[sid] = ReceivedMessages(task, value, MessageQueue(), amqp)
+
+        queue = MessageQueue(const.DEFAULT_CREDIT, const.QUEUE_SIZE_THRESHOLD)
+        self._subscriptions[sid] = ReceivedMessages(queue, value, amqp)
         return await self.send_request(const.KEY_SUBSCRIBE, data)
 
-    async def read_stream(self, subscription_id: int, amqp: bool) -> MessageQueue:
+    async def read_stream(self, subscription_id: int) -> deque[MessageCtx]:
         """
         Read RabbitMQ stream messages for the subscription id.
 
         :param subscription_id: Subscription id for RabbitMQ stream.
         """
         rm = self._subscriptions[subscription_id]
-        task = rm.task
         queue = rm.queue
-        if task.done() and queue:
-            # because of subscription credit, the message queue can get
-            # updated already, do not create a new task, just return the
-            # messages
-            return queue
-        elif task.done():
-            task = self._loop.create_future()
-            self._subscriptions[subscription_id] = ReceivedMessages(task, None, queue, amqp)
 
-        await task
-        return queue
+        # credit of the subscription is usually renewed in
+        # process_message_delivery method; it might not happen there if
+        # message queue threshold is breached, so check if it needs to be
+        # updated before we start waiting for new messages
+        if queue.needs_credit:
+            self.renew_credit(subscription_id, queue)
+
+        await queue.wait()
+        self._subscriptions[subscription_id] = dtc.replace(rm, offset=None)
+        return queue.data
 
     async def unsubscribe(self, subscription_id: int) -> tp.Any:
         """
@@ -425,6 +447,15 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         data = codec.encode_stream(stream)
         return await self.send_request(const.KEY_CREATE_STREAM, data)
 
+    async def send_delete_stream(self, stream: str) -> tp.Any:
+        """
+        Delete RabbitMQ stream.
+
+        :param stream: Stream name.
+        """
+        data = codec.encode_string(stream)
+        return await self.send_request(const.KEY_DELETE_STREAM, data)
+
     async def query_stream_metadata(self, stream: str) -> tp.Any:
         """
         Query RabbitMQ stream metadata.
@@ -578,6 +609,16 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
             self.heartbeat, self.timestamp_callback
         )
 
+    def renew_credit(self, subscription_id: int, queue: MessageQueue) -> None:
+        """
+        Renew credit of RabbitMQ stream subscription if necessary.
+
+        :param subscription_id: RabbitMQ stream subscription id.
+        :param queue: RabbitMQ stream message queue.
+        """
+        self.send_credit(subscription_id, queue.default_credit)
+        queue.reset_credit()
+
     def data_received(self, chunk: bytes) -> None:
         self._timestamp = self._loop.time()
         for start, key in self.decoder.commands(chunk):
@@ -761,11 +802,11 @@ class RabbitMQStreamsProtocol(asyncio.Protocol):
         sid = data[offset]
         if sid in self._subscriptions:
             rm = self._subscriptions[sid]
-            task = rm.task
-            codec.decode_messages(data, offset + 1, rm.offset, rm.queue, rm.amqp)
+            queue = rm.queue
+            codec.decode_messages(data, offset + 1, rm.offset, queue, rm.amqp)
 
-            if not task.done():
-                task.set_result(None)
+            if queue.set():
+                self.renew_credit(sid, queue)
         else:
             logger.warning(
                 'subscription not found, id={}'.format(sid)
