@@ -22,11 +22,6 @@ Basic types for RabbitMQ Streams implementation.
 """
 
 import typing as tp
-from collections import deque
-
-from ..amqp import MessageCtx
-
-MessageQueue = deque[MessageCtx]
 
 class PublishError(tp.NamedTuple):
     """
