@@ -21,7 +21,7 @@ import asyncio
 import logging
 import typing as tp
 from collections.abc import Awaitable
-from itertools import chain as concatv
+from itertools import chain as concatv, islice
 
 logger = logging.getLogger(__name__)
 
@@ -32,6 +32,19 @@ Predicate = tp.Callable[[Exception], bool]
 def default_predicate(ex: Exception) -> bool:
     return True
 
+def partition(items: tp.Iterable[T], size: int) -> tp.Iterable[tuple[T, ...]]:
+    """
+    Partition collection of items into chunks.
+
+    Based on a receipe presented in `itertools` module documentation.
+
+    :param items: Items to partition into chunks.
+    :param size: Chunk size.
+    """
+    it = iter(items)
+    while (chunk := tuple(islice(it, size))):
+        yield chunk
+
 def suppress(
         ex_cls: type[Exception],
         *,
