@@ -24,12 +24,13 @@ commands.
 
 import struct
 import typing as tp
+from collections import deque
 
 from rbfly.amqp import MessageCtx
 from rbfly.streams import codec
+from rbfly.streams._mqueue import MessageQueue
 from rbfly.streams.offset import Offset
 from rbfly.streams.protocol import RabbitMQStreamsProtocol
-from rbfly.streams.types import MessageQueue
 
 import pytest
 from unittest import mock
@@ -193,9 +194,9 @@ def test_decode_delivered_message(
     """
     Test decoding delivered messages.
     """
-    result = MessageQueue()
+    result = MessageQueue(2, 16)
     codec.decode_messages(data, 0, offset, result, True)
-    assert list(result) == expected
+    assert list(result.data) == expected
 
 @pytest.mark.parametrize('offset, expected', OFFSET_DATA)
 def test_encode_offset(offset: Offset, expected: bytes) -> None:
